@@ -177,7 +177,7 @@ def plot_pulls(**kwargs):
               help='whether to adjust the fit range to median limit +- 5 sigma for observed fit')
 @click.option('--do_tilde/--no_tilde', default=True, show_default=True,
               help='bound mu at zero if true and do the \tilde{q}_{mu} asymptotics')
-@click.option('--predictive_fit/--no_predictive_fit', default=True, show_default=True,
+@click.option('--predictive_fit/--no_predictive_fit', default=False, show_default=True,
               help='extrapolate best fit nuisance parameters based on previous fit results')
 @click.option('--do_better_bands/--skip_better_bands', default=True, show_default=True,
               help='evaluate asymptotic CLs limit for various sigma bands')
