@@ -6,13 +6,18 @@ import click
 @click.option('-i', '--input_file', "filename", required=True, 
               help='Path to the input workspace file.')
 @click.option('-o', '--outname', default='fit_result.json', show_default=True,
-              help='Name of output.')
+              help='Name of output file.')
 @click.option('--display/--no-display', default=True, show_default=True,
               help='Display fit result.')
 @click.option('--save/--no-save', "save_result", default=False, show_default=True,
               help='Save fit result')
 @click.option('--save_log/--skip_log', default=False, show_default=True,
               help='Save log file')
+@click.option('--save_ws', default=None, show_default=True,
+              help='Save fitted workspace to a given path.')
+@click.option('--save_snapshot', default=None, show_default=True,
+              help='Save fitted values of all variables as a snapshot and restore all variables to '
+              'their initial values. Should be used together with --save_ws.')
 @click.option('--outdir', default="pulls", show_default=True,
               help='Output directory for pulls output')
 @click.option('--export_as_np_pulls/--skip_export_as_np_pulls', default=False, show_default=True,
@@ -43,6 +48,8 @@ import click
               help='Maximum number of retries upon a failed fit')
 @click.option('--strategy', type=int, default=1, show_default=True,
               help='Default minimization strategy')
+@click.option('--hesse/--no-hesse', default=False, show_default=True,
+              help='Evaluate errors using Hesse.')
 @click.option('--print_level', type=int, default=-1, show_default=True,
               help='Minimizer print level')
 @click.option('--fix-cache/--no-fix-cache', default=True, show_default=True,
@@ -57,6 +64,11 @@ import click
               help='Optimize constant terms')
 @click.option('--offset/--no-offset', default=True, show_default=True,
               help='Offset likelihood')
+@click.option('--batch_mode/--no-batch', default=False, show_default=True,
+              help='Batch mode when evaluating likelihood')
+@click.option('--int_bin_precision', type=float, default=-1., show_default=True,
+              help='Integrate the PDF over the bins instead of using the probability '
+                   'density at the bin centre')
 @click.option('-v', '--verbosity', default='INFO', show_default=True,
               type=click.Choice(["DEBUG", "INFO", "WARNING", "ERROR"], case_sensitive=False),
               help='Verbosity level.')
@@ -66,7 +78,7 @@ def likelihood_fit(**kwargs):
     """
     _kwargs = {}
     for arg_name in ["outname", "save_log", "display", "save_result",
-                     "export_as_np_pulls", "outdir"]:
+                     "export_as_np_pulls", "outdir", "save_ws", "save_snapshot"]:
         _kwargs[arg_name] = kwargs.pop(arg_name)
     _init_kwargs = {}
     for arg_name in ["filename", "data_name", "verbosity"]:
@@ -134,7 +146,14 @@ def likelihood_fit(**kwargs):
             outpath = os.path.join(outdir, f"{np_name}.json")
             with open(outpath, "w") as outfile:
                 json.dump({"nuis": data}, outfile, indent=2)
-                
+    if _kwargs["save_ws"] is not None:
+        filename = _kwargs["save_ws"]
+        if _kwargs["save_snapshot"] is not None:
+            snapshot_name = _kwargs["save_snapshot"]
+            analysis.save_snapshot(snapshot_name)
+            analysis.load_snapshot(analysis.kInitialSnapshotName)
+        analysis.save(filename)
+        
 @click.command(name='np_correlation')
 @click.option('-i', '--input_file', "filename", required=True, 
               help='Path to the input workspace file.')
@@ -193,6 +212,11 @@ def likelihood_fit(**kwargs):
               help='Optimize constant terms')
 @click.option('--offset/--no-offset', default=True, show_default=True,
               help='Offset likelihood')
+@click.option('--batch_mode/--no-batch', default=False, show_default=True,
+              help='Batch mode when evaluating likelihood')
+@click.option('--int_bin_precision', type=float, default=-1., show_default=True,
+              help='Integrate the PDF over the bins instead of using the probability '
+                   'density at the bin centre')
 @click.option('-v', '--verbosity', default='INFO', show_default=True,
               type=click.Choice(["DEBUG", "INFO", "WARNING", "ERROR"], case_sensitive=False),
               help='Verbosity level.')
@@ -211,6 +235,7 @@ def np_correlation(**kwargs):
     _init_kwargs['poi_name'] = []
     from quickstats.components import AnalysisBase   
     analysis = AnalysisBase(**_init_kwargs)
+    analysis.minimizer.configure(hesse=True)
     analysis.nll_fit(mode=3)
     fit_result = analysis.roofit_result
     basename = os.path.splitext(_kwargs['basename'])[0]
