@@ -128,10 +128,6 @@ class ExtendedModel(AbstractObject):
             result = load_macro(extension)
             if (result is not None) and hasattr(ROOT, extension):
                 self.stdout.info(f'INFO: Loaded extension module "{extension}"')
-            
-    @staticmethod
-    def load_RooFitObjects():
-        load_macro('RooFitObjects')
         
     @semistaticmethod
     def modify_interp_codes(self, ws, interp_code, classes=None):
@@ -767,7 +763,6 @@ class ExtendedModel(AbstractObject):
         
     @staticmethod
     def get_object_map(object_dict:Dict, object_name:str):
-        ExtendedModel.load_RooFitObjects()
         if object_name not in ["RooDataSet", "RooAbsPdf"]:
             raise ValueError(f"unsupported object \"{object_name}\"")
         object_map = ROOT.std.map(f"string, {object_name}*")()
@@ -779,7 +774,6 @@ class ExtendedModel(AbstractObject):
     
     @staticmethod
     def get_dataset_map(dataset_dict:Dict):
-        ExtendedModel.load_RooFitObjects()
         dsmap = ROOT.std.map('string, RooDataSet*')()
         dsmap.keepalive = list()
         for c, d in dataset_dict.items():
