@@ -14,13 +14,12 @@ from quickstats import semistaticmethod
 from quickstats.components import AbstractObject, ExtendedModel
 from quickstats.utils.io import Verbosity
 from quickstats.utils.xml_tools import TXMLTree
-from quickstats.utils.root_utils import load_macro
 from quickstats.utils.common_utils import remove_list_duplicates, format_delimiter_enclosed_text
 from quickstats.maths.numerics import is_float, pretty_value
 
-from quickstats.components.workspaces import Sample, Systematic, AsimovHandler
+from quickstats.components.workspaces import XMLWSBase, Sample, Systematic, AsimovHandler
     
-class XMLWSBuilder(AbstractObject):
+class XMLWSBuilder(XMLWSBase):
     
     EPSILON = 1e-6
     
@@ -94,23 +93,17 @@ class XMLWSBuilder(AbstractObject):
         "save_snapshot": "savesnapshot"
     }
     
-    def __init__(self, filename:str, basedir:Optional[str]=None,
+    def __init__(self, source:str, basedir:Optional[str]=None,
                  use_binned:bool=False,
                  data_storage_type:str="vector",
                  minimizer_config:Optional[Dict]=None,
                  verbosity:Optional[Union[int, str]]="INFO"):
-        super().__init__(verbosity=verbosity)
-        self.filename = filename
-        if basedir is None:
-            self.basedir = os.path.dirname(self.filename)
-        else:
-            self.basedir = basedir
+        super().__init__(source=source, basedir=basedir, verbosity=verbosity)
         self.use_binned = use_binned
         self.data_storage_type = data_storage_type
         self.minimizer_config = minimizer_config
         self.load_extension()
-        self.suppress_roofit_message()
-        self.initialize(filename=filename)
+        self.initialize(source=source)
     
     @staticmethod
     def set_data_storage_type(data_storage_type:str="vector"):
@@ -122,17 +115,9 @@ class XMLWSBuilder(AbstractObject):
             ROOT.RooAbsData.setDefaultStorageType(ROOT.RooAbsData.StorageType.Composite)            
         else:
             raise RuntimeError(f"unknown data storage type `{data_storage_type}`")
-    
-    def suppress_roofit_message(self):
-        ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.NumIntegration)
-        ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.Fitting)
-        ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.Minimization)
-        ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.InputArguments)
-        ROOT.RooMsgService.instance().getStream(1).removeTopic(ROOT.RooFit.Eval)
-        ROOT.RooMsgService.instance().setGlobalKillBelow(ROOT.RooFit.ERROR)
-        
-    def initialize(self, filename:str):
-        self.core_config = self.parse_core_xml(filename)
+  
+    def initialize(self, source:str):
+        self.core_config = self.parse_core_xml(source)
         self._print_core_xml_summary()
         self.cat_config = {}
         self.cat_ws = {}
@@ -145,13 +130,6 @@ class XMLWSBuilder(AbstractObject):
         self.set_data_storage_type(self.data_storage_type)
         self.comb_ws = ROOT.RooWorkspace(ws_name)
         self.model_config = ROOT.RooStats.ModelConfig(mc_name, self.comb_ws)
-        
-    @semistaticmethod
-    def load_extension(self):
-        for extension in ['FlexibleInterpVarMkII', 'RooTwoSidedCBShape']:
-            result = load_macro(extension)
-            if (result is not None) and hasattr(ROOT, extension):
-                self.stdout.info(f'INFO: Loaded extension module "{extension}"')
     
     @semistaticmethod
     def _xml_to_dict(self, filename:str):
@@ -161,14 +139,6 @@ class XMLWSBuilder(AbstractObject):
             raise RuntimeError(f"failed to load xml document `{filename}`")
         return xml_dict
     
-    def get_relpath(self, filename:str):
-        if filename.startswith("/") or filename.startswith("~"):
-            return filename
-        return os.path.join(self.basedir, filename)
-    
-    def check_file_exist(self, filename:str):
-        return os.path.exists(self.get_relpath(filename))
-    
     def parse_core_xml(self, filename:str):
         self.stdout.info(f"INFO: Parsing file `{filename}`")
         # load xml file
@@ -441,7 +411,7 @@ class XMLWSBuilder(AbstractObject):
         
         self.current_category = category_name
         category_type = attributes['Type'].lower()
-        luminosity = float(self._get_node_attrib(cat_xml, "Lumi", True, "-1"))
+        luminosity = float(self._get_node_attrib(cat_xml, "Lumi", False, "-1"))
         
         self.cat_config[category_name] = {
             'input_file': filename,
@@ -1001,34 +971,7 @@ class XMLWSBuilder(AbstractObject):
         else:
             object_type = "defined"
             object_name = expr
-        return object_name, object_type        
-
-    def import_expression(self, ws:ROOT.RooWorkspace, object_expr:str, check_defined:bool=False):
-        object_name, object_type = self._get_object_name_and_type_from_expr(object_expr)
-        # throw error when attemping to access undefined object
-        if object_type == "defined":
-            if (not ws.arg(object_name)):
-                raise RuntimeError(f"object `{object_name}` does not exist")
-            else:
-                return object_name
-        # throw error? when attemping to define already existing object
-        if check_defined:
-            if ws.arg(object_name):
-                self.stdout.debug(f"REGTEST: object {object_name} already exists")
-                return object_name
-        self.stdout.debug(f"REGTEST: Generating {object_type} {object_expr}")
-        status = ws.factory(object_expr)
-        if not status:
-            raise RuntimeError(f"object creation from expression `{object_expr}` had failed")
-        return object_name
-    
-    def import_expressions(self, ws:ROOT.RooWorkspace, object_expr_list:List[str],
-                       check_defined:bool=False):
-        object_names = []
-        for object_expr in object_expr_list:
-            object_name = self.import_expression(ws, object_expr, check_defined=check_defined)
-            object_names.append(object_name)
-        return ",".join(object_names)
+        return object_name, object_type
             
     def import_uncert_expr(self, ws:ROOT.RooWorkspace, expr:str, var_name:str, prefix:str):
         if is_float(expr):
@@ -1036,25 +979,7 @@ class XMLWSBuilder(AbstractObject):
             return self.import_expression(ws, new_expr)
         else:
             return self.import_expression(ws, expr)
-        
-    def import_object(self, ws:ROOT.RooWorkspace, obj, *args, silent:bool=True):
-        if silent:
-            getattr(ws, "import")(obj, *args, ROOT.RooFit.Silence())        
-        else:
-            getattr(ws, "import")(obj, *args)        
-    
-    @staticmethod
-    def _get_node_attrib(node:Dict, attrib_name:str, allow_empty:bool=False, default:str=""):
-        attrib_val = node['attrib'].get(attrib_name, None)
-        if (attrib_val is None):
-            if allow_empty:
-                attrib_val = default
-            else:
-                raise RuntimeError(f"attribute `{attrib_name}` not found in the "
-                                   f"node `{node['tag']}`")
-        attrib_val = attrib_val.strip()
-        return attrib_val    
-    
+
     def _translate_keyword(self, expr:str):
         category_name = self.current_category
         observable_name = self.cat_config[category_name]['observable']['name']
@@ -1076,8 +1001,8 @@ class XMLWSBuilder(AbstractObject):
         return expr
        
     def _translate_node_attrib(self, node:Dict, attrib_name:str, process:str="",
-                               allow_empty:bool=False, default:str=""):
-        expr = self._get_node_attrib(node, attrib_name, allow_empty, default)
+                               required:bool=True, default:str=""):
+        expr = self._get_node_attrib(node, attrib_name, required, default)
         expr = self._translate_keyword(expr)
         if (self.KEYWORDS['process'] in expr):
             if process == "":
@@ -1167,7 +1092,7 @@ class XMLWSBuilder(AbstractObject):
         syst.name = self._translate_node_attrib(node, "Name", domain)
         # if the process of the systematic is specified, use the 
         # specified process. Otherwise use the default one
-        syst.process = self._translate_node_attrib(node, "Process", domain, True, "")
+        syst.process = self._translate_node_attrib(node, "Process", domain, False, "")
         # common systematics
         if domain == self.KEYWORDS['allproc']:
             # if a process name is specified, use it as domain name and 
@@ -1190,17 +1115,17 @@ class XMLWSBuilder(AbstractObject):
             else:
                 syst.process = f"{domain}_{syst.process}"
         syst.whereto = self._get_node_attrib(node, "WhereTo")
-        syst.nominal = float(self._get_node_attrib(node, "CentralValue"))
+        syst.nominal = self._get_node_attrib(node, "CentralValue", dtype="float")
         syst.constr_term = self._get_node_attrib(node, "Constr")
         if (syst.nominal <= 0.) and \
         (syst.constr_term in [self.KEYWORDS['logn'], self.KEYWORDS['asym']]):
             raise RuntimeError(f"failed to parse systematic {syst.name}:"
                                f" constraint term {syst.constr_term} received"
                                f" negative central value ({syst.nominal})")
-        uncert_expr = self._get_node_attrib(node, "Mag").split(",")
+        uncert_expr = self._get_node_attrib(node, "Mag", dtype="str_list")
         uncert_val  = [float(v) if is_float(v) else None for v in uncert_expr]
         uncerts_sign = [-1 if (v is not None) and (v < 0) else +1 for v in uncert_val]
-        beta = float(self._get_node_attrib(node, "Beta", True, "1"))
+        beta = self._get_node_attrib(node, "Beta", False, "1", dtype="float")
         beta *= uncerts_sign[0]
         syst.beta = beta
         
@@ -1283,7 +1208,7 @@ class XMLWSBuilder(AbstractObject):
             sample.norm_factors.append(factor)
         elif factor_type == "shape":
             sample.shape_factors.append(factor)
-        is_correlated = int(self._get_node_attrib(node, "Correlate", True, "0"))
+        is_correlated = self._get_node_attrib(node, "Correlate", False, "0", dtype="int")
         if is_correlated:
             cat_config = self.get_active_category_config()
             item_name = self._get_object_name_and_type_from_expr(factor)
@@ -1313,9 +1238,10 @@ class XMLWSBuilder(AbstractObject):
             'nbins': observable_nbins
         }
         
-        data_filename = self._translate_node_attrib(node, "InputFile",
-                                                    "", is_counting, "")
-        data_filetype = self._get_node_attrib(node, "FileType", True,
+        data_filename = self._translate_node_attrib(node, "InputFile", "",
+                                                    required=not is_counting,
+                                                    default="")
+        data_filetype = self._get_node_attrib(node, "FileType", False,
                                               self.KEYWORDS['ascii'])
         data_filetype = data_filetype.lower()
         data_config = {}
@@ -1328,15 +1254,17 @@ class XMLWSBuilder(AbstractObject):
             else:
                 data_config['treename'] = self._translate_node_attrib(node, "TreeName")
                 data_config['varname'] = self._translate_node_attrib(node, "VarName")
-                data_config['cut'] = self._translate_node_attrib(node, "Cut", "", True, "")
-        inject_ghost = int(self._get_node_attrib(node, "InjectGhost", True, "0"))
+                data_config['cut'] = self._translate_node_attrib(node, "Cut", "",
+                                                                 required=False,
+                                                                 default="")
+        inject_ghost = self._get_node_attrib(node, "InjectGhost", False, "0", dtype="int")
         data_config['inject_ghost'] = inject_ghost
         if (is_counting and data_filename == ""):
             data_config['num_data'] = self._get_node_attrib(node, "NumData")
         else:
             data_config['num_data'] = -1
-        data_config['scale_data'] = float(self._get_node_attrib(node, "ScaleData", True, "1"))
-        blind_range = self._get_node_attrib(node, "BlindRange", True, None)
+        data_config['scale_data'] = self._get_node_attrib(node, "ScaleData", False, "1", dtype="float")
+        blind_range = self._get_node_attrib(node, "BlindRange", False, None)
         if blind_range:
             blind_range = blind_range.split(",")
             if (len(blind_range) != 2) or (not is_float(blind_range[0])) or (not is_float(blind_range[1])):
@@ -1367,15 +1295,16 @@ class XMLWSBuilder(AbstractObject):
         sample = Sample()
         sample.process = self._translate_node_attrib(node, "Name")
         sample.input_file = self._translate_node_attrib(node, "InputFile", sample.process,
-                                                        is_counting, "")
+                                                        required=not is_counting,
+                                                        default="")
         syst_groups = self._translate_node_attrib(node, "ImportSyst", sample.process,
-                                                  True,  self.KEYWORDS['self'])
+                                                  required=False, default=self.KEYWORDS['self'])
         syst_groups = [s for s in syst_groups.split(",") if s != sample.process]
         syst_groups = remove_list_duplicates(syst_groups)
         sample.syst_groups = syst_groups
         
         if cat_config['luminosity'] > 0:
-            is_multiply_lumi = int(self._get_node_attrib(node, "MultiplyLumi", True, "1"))
+            is_multiply_lumi = self._get_node_attrib(node, "MultiplyLumi", False, "1", dtype="int")
         else:
             is_multiply_lumi = 0
         
@@ -1393,13 +1322,14 @@ class XMLWSBuilder(AbstractObject):
         }
         
         for key, name in norm_factor_names.items():
-            norm_value = self._translate_node_attrib(node, name, sample.process, True,  "")
+            norm_value = self._translate_node_attrib(node, name, sample.process,
+                                                     required=False, default="")
             if norm_value != "":
                 norm_name = self.KEYWORDS[f'{key}_name']
                 sample.norm_factors.append(f"{norm_name}_{sample.process}[{norm_value}]")
                 
         sample.share_pdf_group = self._translate_node_attrib(node, "SharePdf", sample.process,
-                                                             True,  "")
+                                                             required=False, default="")
         subnodes = node['children']
         self.stdout.debug(f"REGTEST: Reading sample `{sample.process}`")
         self.read_sample_subnode(subnodes, sample)
@@ -1564,7 +1494,7 @@ class XMLWSBuilder(AbstractObject):
         cat_config = self.get_active_category_config()
         tag_name = sample.get_tag_name()
         obs_name = cat_config['observable']['name']
-        cache_binning = int(self._get_node_attrib(root_node, "CacheBinning", True, "-1"))
+        cache_binning = self._get_node_attrib(root_node, "CacheBinning", False, "-1", dtype="int")
         obs = ws.var(obs_name)
         if cache_binning > 0:
             if not obs:
@@ -1627,7 +1557,7 @@ class XMLWSBuilder(AbstractObject):
                 if not var:
                     raise RuntimeError(f"no variable named `{var_name}` found im the workspace `{filepath}`")
                 var.setConstant(1)
-                var_val = self._get_node_attrib(node, "Value", True, "")
+                var_val = self._get_node_attrib(node, "Value", False, "")
                 if var_val != "":
                     var.setVal(float(var))
                 self.stdout.debug(f"REGTEST: Fixed variable `{var_name}` at `{var.getVal()}")
@@ -1640,9 +1570,9 @@ class XMLWSBuilder(AbstractObject):
             elif name == "ExtSyst":
                 # adding external systematics
                 np_name = self._translate_node_attrib(node, "NPName")
-                glob_name = self._translate_node_attrib(node, "GOName", "", True, "")
-                constr_name = self._translate_node_attrib(node, "ConstrName", 
-                                                          "", glob_name=="", "")
+                glob_name = self._translate_node_attrib(node, "GOName", "", False, "")
+                constr_name = self._translate_node_attrib(node, "ConstrName", "",
+                                                          required=glob_name!="", default="")
                 np_list.append(np_name)
                 glob_list.append(glob_name)
                 constr_list.append(constr_name)
@@ -1718,7 +1648,7 @@ class XMLWSBuilder(AbstractObject):
         filename = self._translate_node_attrib(root_node, "Input")
         filepath = self.get_relpath(filename)
         histname = self._translate_node_attrib(root_node, "ModelName")
-        rebin = int(self._get_node_attrib(root_node, "Rebin", True, "-1"))
+        rebin = self._get_node_attrib(root_node, "Rebin", False, "-1", dtype="int")
         self.stdout.debug(f"REGTEST: Creating histogram pdf from `{filepath}`")
         h = self.get_histogram(filename, histname)
         if rebin > 0:
