@@ -40,8 +40,9 @@ class XMLWSModifier(XMLWSBase):
     def initialize(self, source:Union[str, Dict]):
         self.config = None
         self.actions = {
-            'define': [],
             'map': [],
+            'define': [],
+            'redefine': [],
             'asimov': [],
             'constraint': []
         }
@@ -67,28 +68,31 @@ class XMLWSModifier(XMLWSBase):
     def parse_config_dict(self, source:str):
         _actions = source.pop("actions", {})
         config = {
-            "input_file"       : source.pop("input_file"),
-            "output_file"      : source.pop("output_file"),
-            "model_name"       : source.pop("model_name", None),
-            "workspace_name"   : source.pop("workspace_name", None),
-            "model_config_name": source.pop("model_config_name", None),
-            "data_name"        : source.pop("data_name", "combData"),
-            "poi_names"        : source.pop("poi_names", None),
-            "snapshot_nuis"    : source.pop("snapshot_nuis", []),
-            "snapshot_globs"   : source.pop("snapshot_globs", []),
-            "snapshot_pois"    : source.pop("snapshot_pois", []),
-            "snapshot_all"     : source.pop("snapshot_all", []),
-            "strict"           : source.pop("strict", True)
+            "input_file"         : source.pop("input_file"),
+            "output_file"        : source.pop("output_file"),
+            "model_name"         : source.pop("model_name", None),
+            "workspace_name"     : source.pop("workspace_name", None),
+            "model_config_name"  : source.pop("model_config_name", None),
+            "data_name"          : source.pop("data_name", "combData"),
+            "poi_names"          : source.pop("poi_names", None),
+            "snapshot_nuis"      : source.pop("snapshot_nuis", []),
+            "snapshot_globs"     : source.pop("snapshot_globs", []),
+            "snapshot_pois"      : source.pop("snapshot_pois", []),
+            "snapshot_all"       : source.pop("snapshot_all", []),
+            "fix_parameters"     : source.pop("fix_parameters", None),
+            "profile_parameters" : source.pop("profile_parameters", None),
+            "strict"             : source.pop("strict", True)
         }
         if source:
             unknown_attributes = list(source)
             self.stdout.warning(f"WARNING: Unknown attributes received: {', '.join(unknown_attributes)}. "
                                 "Ignoring...")
         actions = {
-            "define": _actions.pop("define", []),
-            "constraint": _actions.pop("constraint", []),
-            "map": _actions.pop("map", []),
-            "asimov": _actions.pop("asimov", [])
+            "map"        : _actions.pop("map", []),
+            "define"     : _actions.pop("define", []),
+            "redefine"   : _actions.pop("redefine", []),
+            "constraint" : _actions.pop("constraint", []),
+            "asimov"     : _actions.pop("asimov", [])
         }
         _rename = _actions.pop("rename", {})
         rename = {
@@ -128,6 +132,8 @@ class XMLWSModifier(XMLWSBase):
             'snapshot_globs'   : self._get_node_attrib(root, "SnapshotGO", required=False, default=[], dtype="str_list"),
             'snapshot_pois'    : self._get_node_attrib(root, "SnapshotPOI", required=False, default=[], dtype="str_list"),
             'snapshot_all'     : self._get_node_attrib(root, "SnapshotAll", required=False, default=[], dtype="str_list"),
+            "fix_parameters"     : self._get_node_attrib(root, "FixParameters", required=False, default=None),
+            "profile_parameters" : self._get_node_attrib(root, "ProfileParameters", required=False, default=None),
             'strict'           : self._get_node_attrib(root, "Strict", required=False, default="true", dtype="bool")
         }
         # for compatibility with older version of workspaceCombiner
@@ -238,6 +244,7 @@ class XMLWSModifier(XMLWSBase):
         filename = self.config['input_file']
         ws_name  = self.config['workspace_name']
         mc_name  = self.config['model_config_name']
+        self.stdout.info(f"INFO: Begin modification of the workspace \"{filename}\".")
         model    = ExtendedModel(filename, ws_name=ws_name, mc_name=mc_name,
                                  data_name=None, verbosity="WARNING")
         ws_orig  = model.workspace
@@ -248,22 +255,29 @@ class XMLWSModifier(XMLWSBase):
         # create temporary workspace
         ws_tmp = ROOT.RooWorkspace(ws_name)
     
-        title_str = format_delimiter_enclosed_text("Step 1: Create new objects", "-")
+        title_str = format_delimiter_enclosed_text("Step 1: Redefine objects", "-")
+        self.stdout.info(title_str)
+        flag = self.redefine_objects(ws_orig, ws_tmp)
+        # nothing is done
+        if not flag:
+            self.stdout.info("INFO: No objects are redefined.")
+        
+        title_str = format_delimiter_enclosed_text("Step 2: Create new objects", "-")
         self.stdout.info(title_str)
         flag = self.implement_external_pdfs(ws_tmp)
         flag |= self.implement_objects(ws_tmp)
         # nothing is done
         if not flag:
-            self.stdout.info("INFO: No new objectes are defined.")
+            self.stdout.info("INFO: No objects are defined.")
         
-        title_str = format_delimiter_enclosed_text("Step 2: Rename variables", "-")
+        title_str = format_delimiter_enclosed_text("Step 3: Rename variables", "-")
         self.stdout.info(title_str)
         rename_map = self.get_rename_map(ws_orig, ws_tmp)
         self.rename_variables(mc_orig, ws_tmp, rename_map)
         if not rename_map:
             self.stdout.info("INFO: No variables are renamed.")
         
-        title_str = format_delimiter_enclosed_text("Step 3: Making output workspace", "-")
+        title_str = format_delimiter_enclosed_text("Step 4: Making output workspace", "-")
         self.stdout.info(title_str)        
         # create final workspace
         if len(self.actions['constraint']) > 0:
@@ -285,6 +299,8 @@ class XMLWSModifier(XMLWSBase):
         self.create_snapshots(ws_orig, mc_orig, ws_final, mc_final, rename_map)
         
         self.generate_asimov(ws_final)
+        
+        self.setup_parameters(ws_final)
 
         outname = self.config['output_file']
         outdir  = os.path.dirname(outname)
@@ -293,6 +309,14 @@ class XMLWSModifier(XMLWSBase):
         ws_final.writeToFile(outname, True)
         self.stdout.info(f"INFO: Saved output workspace as \"{outname}\".")
     
+    def setup_parameters(self, ws:ROOT.RooWorkspace):
+        model = ExtendedModel(ws, data_name=None, verbosity="WARNING")
+        model.stdout.verbosity = "INFO"
+        if self.config["fix_parameters"] is not None:
+            model.fix_parameters(self.config["fix_parameters"])
+        if self.config["profile_parameters"] is not None:
+            model.profile_parameters(self.config["profile_parameters"])
+    
     def import_class_code(self, ws:ROOT.RooWorkspace):
         all_function_class = [i.ClassName().split("::")[-1] for i in ws.allFunctions()]
         all_pdf_class = [i.ClassName().split("::")[-1] for i in ws.allPdfs()]
@@ -560,6 +584,37 @@ class XMLWSModifier(XMLWSBase):
         for dataset in datasets:
             getattr(new_ws, "import")(dataset)
             self.stdout.info(f"INFO: Imported dataset \"{dataset.GetName()}\".")
+            
+    def redefine_objects(self, old_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace):
+        flag = False
+        for i, expr in enumerate(self.actions['redefine']):
+            self.stdout.info(f"INFO: (Item {i}) {expr}")
+            obj_name, obj_type = self._get_object_name_and_type_from_expr(expr)
+            arg = old_ws.arg(obj_name)
+            if not arg:
+                raise RuntimeError(f"object {obj_name} does not exist in the original workspace")
+            if obj_type == WSObjectType.VARIABLE:
+                result = re.search(r"\[(.+)\]", expr)
+                if not result:
+                    raise RuntimeError(f"invalid variable expression {expr}")
+                self.import_expression(new_ws, expr)
+                new_var = new_ws.var(obj_name)
+                tokens = result.group(1).split(",")
+                # only modify the value
+                if len(tokens) == 1:
+                    old_range = arg.getRange()
+                    # need to restore the range
+                    new_var.setRange(old_range[0], old_range[1])
+                # only modify the range
+                elif len(tokens) == 2:
+                    # need to restore the value
+                    new_var.setVal(arg.getVal())
+                # need to restore constant state
+                new_var.setConstant(arg.isConstant())
+            else:
+                self.import_expression(new_ws, expr)
+            flag = True
+        return flag
         
     def implement_objects(self, ws:ROOT.RooWorkspace):
         flag = False
@@ -613,9 +668,9 @@ class XMLWSModifier(XMLWSBase):
                 new_name = tokens[1]
                 if not old_ws.arg(old_name):
                     if strict:
-                        raise RuntimeError(f"object {old_name} (-> {new_name}) does not exist int the old workspace")
+                        raise RuntimeError(f"object {old_name} (-> {new_name}) does not exist in the original workspace")
                     else:
-                        self.stdout.warning(f"object {old_name} does not exist in the old workspace, skipping...")
+                        self.stdout.warning(f"object {old_name} does not exist in the original workspace, skipping...")
                         continue
                 if not new_ws.arg(new_name):
                     raise RuntimeError(f"object {new_name} (<- {old_name}) does not exist in the temporary workspace")
@@ -626,9 +681,9 @@ class XMLWSModifier(XMLWSBase):
         for old_name, new_name in self.actions['rename']['variable'].items():
             if not old_ws.arg(old_name):
                 if strict:
-                    raise RuntimeError(f"object {old_name} (-> {new_name}) does not exist int the old workspace")
+                    raise RuntimeError(f"object {old_name} (-> {new_name}) does not exist int the original workspace")
                 else:
-                    self.stdout.warning(f"object {old_name} does not exist in the old workspace, skipping...")
+                    self.stdout.warning(f"object {old_name} does not exist in the original workspace, skipping...")
                     continue
             if old_name in rename_map:
                 raise RuntimeError(f"object {old_name} is renamed more than once")
