@@ -6,7 +6,6 @@ import ROOT
 
 from quickstats import semistaticmethod
 from quickstats.interface.cppyy.vectorize import as_np_array
-from quickstats.utils.root_utils import load_macro
 
 class RooDataSet:
     
@@ -105,7 +104,6 @@ class RooDataSet:
     
     @staticmethod
     def get_dataset_map(dataset_dict:Dict):
-        load_macro('RooFitObjects')
         dsmap = ROOT.std.map('string, RooDataSet*')()
         dsmap.keepalive = list()
         for c, d in dataset_dict.items():
