@@ -20,7 +20,8 @@ class Likelihood2DPlot(AbstractPlot):
             'markersize' : 20,
             'color' : '#E9F1DF',
             'markeredgecolor' : 'black'
-        }
+        },
+        'cmap': 'GnBu'
     }
     # https://pdg.lbl.gov/2018/reviews/rpp2018-rev-statistics.pdf#page=31
     likelihood_label_threshold = {
@@ -59,8 +60,9 @@ class Likelihood2DPlot(AbstractPlot):
             return list(self.data_map)
         
     def draw_single_data(self, ax, data:pd.DataFrame, 
+                         xattrib:str, yattrib:str, zattrib:str='qmu',
                          styles:Optional[Dict]=None,
-                         xattrib:str='klambda', yattrib:str='k2v', zattrib:str='nll', clabel_size=None):
+                         clabel_size=None, show_colormesh=False):
         colors = ['k'] + list(self.config['sigma_colors'])
         levels = [0] + [self.likelihood_label_threshold[key][1] for key in self.config['sigma_levels']]
         x = data[xattrib]
@@ -70,6 +72,9 @@ class Likelihood2DPlot(AbstractPlot):
         X, Y = np.meshgrid(X_unique, Y_unique)
         Z = self.data_map.pivot_table(index=xattrib, columns=yattrib, values=zattrib).T.values - self.data_map[zattrib].min()
 
+        if show_colormesh:
+            cmap = self.config['cmap']
+            ax.pcolormesh(X, Y, Z, cmap=cmap, shading='auto')
         cp = ax.contour(X, Y, Z, levels=levels, colors=colors, linewidths=2)
         if clabel_size is not None:
             ax.clabel(cp, inline=True, fontsize=clabel_size)
@@ -80,36 +85,17 @@ class Likelihood2DPlot(AbstractPlot):
 
         return custom_handles
     
-    def draw(self, xattrib:str='klambda', yattrib:str='k2v', zattrib:str='nll', xlabel:Optional[str]="$\kappa_\lambda$", 
-             ylabel:Optional[str]="$\kappa_{2v}$", zlabel:Optional[str]="$-2\Delta ln(L)$",
-             ymax:float=5, ymin:float=-5, xmin:Optional[float]=-10, xmax:Optional[float]=10, clabel_size=None, draw_sm_line:bool=False):
+    def draw(self, xattrib:str, yattrib:str, zattrib:str='qmu', xlabel:Optional[str]="", 
+             ylabel:Optional[str]="", zlabel:Optional[str]="$-2\Delta ln(L)$",
+             ymax:float=5, ymin:float=-5, xmin:Optional[float]=-10, xmax:Optional[float]=10,
+             clabel_size=None, draw_sm_line:bool=False, show_colormesh=False):
         ax = self.draw_frame()
         if isinstance(self.data_map, pd.DataFrame):
-            self.draw_single_data(ax, self.data_map, self.styles_map,
-                                  xattrib=xattrib, yattrib=yattrib, clabel_size=clabel_size)
+            self.draw_single_data(ax, self.data_map, xattrib=xattrib, yattrib=yattrib,
+                                  zattrib=zattrib, styles=self.styles_map,
+                                  clabel_size=clabel_size, show_colormesh=show_colormesh)
         elif isinstance(self.data_map, dict):
             assert(0), "not implemented"
-            if self.styles_map is None:
-                styles_map = {k:None for k in self.data_map}
-            else:
-                styles_map = self.styles_map
-            if self.label_map is None:
-                label_map = {k:k for k in data_map}
-            else:
-                label_map = self.label_map
-            handles = {}
-            for key in self.data_map:
-                data = self.data_map[key]
-                styles = styles_map.get(key, None)
-                label = label_map.get(key, "")
-                handle = self.draw_single_data(ax, data, styles, 
-                                               label=label,
-                                               xattrib=xattrib,
-                                               yattrib=yattrib,
-                                               clabel_size=clabel_size
-                                               )
-                handles[key] = handle
-            self.update_legend_handles(handles)
         else:
             raise ValueError("invalid data format")
 
@@ -130,10 +116,10 @@ class Likelihood2DPlot(AbstractPlot):
 
         handles, labels = self.get_legend_handles_labels()
         ax.legend(handles, labels, **self.styles['legend'])
-        ax.set_ylim([ymin, ymax])
-        ax.set_xlim([xmin, xmax])
+        
 
         self.draw_axis_components(ax, xlabel=xlabel, ylabel=ylabel)
+        self.set_axis_range(ax, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax)
         
         return ax
 
