@@ -41,7 +41,7 @@ def score_distribution_plot(dfs:Dict[str, pd.DataFrame], hist_options:Dict[str,
     ax.set_xlabel(xlabel, **styles['xlabel'])
     ax.set_ylabel(ylabel.format(bin_width=bin_width), **styles['ylabel'])
     handles, labels = ax.get_legend_handles_labels()
-    new_handles = [Line2D([], [], c=h.get_edgecolor(), **styles['legend_Line2D'])
+    new_handles = [Line2D([], [], c=h.get_edgecolor(), linestyle=h.get_linestyle(), **styles['legend_Line2D'])
                    if isinstance(h, Polygon) else h for h in handles]
     ax.legend(handles=new_handles, labels=labels, **styles['legend'])
     if boundaries is not None:
