@@ -9,7 +9,6 @@ import inspect
 import datetime
 import functools
 import collections.abc
-from concurrent.futures  import ProcessPoolExecutor
 
 import numpy as np
 
@@ -20,7 +19,7 @@ def get_cpu_count():
     return os.cpu_count()
 
 def parallel_run(func, *iterables, max_workers):
-
+    from concurrent.futures  import ProcessPoolExecutor
     with ProcessPoolExecutor(max_workers) as executor:
         result = executor.map(func, *iterables)
 
@@ -117,8 +116,8 @@ class NpEncoder(json.JSONEncoder):
     
 def update_nested_dict(d:Dict, u:Dict):
     for k, v in u.items():
-        if isinstance(v, collections.abc.Mapping):
-            d[k] = update_nested_dict(d.get(k, {}), v)
+        if isinstance(d.get(k, None), collections.Mapping) and isinstance(v, collections.Mapping):
+            d[k] = update_nested_dict(d[k], v)
         else:
             d[k] = v
     return d
