@@ -1,4 +1,4 @@
-from typing import Union, Optional, Dict, List
+from typing import Union, Optional, Dict, List, Set
 
 import ROOT
 
@@ -10,22 +10,34 @@ def copy_attributes(source:"ROOT.RooAbsArg", target:"ROOT.RooAbsArg"):
     for attrib in source.stringAttributes():
         target.setStringAttribute(attrib.first, attrib.second)
         
-def get_variable_attributes(variable:ROOT.RooRealVar):
-    attributes = {
-        'name' : variable.GetName(),
-        'value': variable.getVal(),
-        'error': variable.getError(),
-        'min'  : variable.getMin(),
-        'max'  : variable.getMax(),
-        'is_constant': variable.isConstant()
-    }
+def get_variable_attributes(variable:ROOT.RooRealVar, asym_error:bool=False):
+    if asym_error:
+        attributes = {
+            'name' : variable.GetName(),
+            'value': variable.getVal(),
+            'errorlo': variable.getErrorLo(),
+            'errorhi': variable.getErrorHi(),
+            'min'  : variable.getMin(),
+            'max'  : variable.getMax(),
+            'is_constant': variable.isConstant()
+        }
+    else:
+        attributes = {
+            'name' : variable.GetName(),
+            'value': variable.getVal(),
+            'error': variable.getError(),
+            'min'  : variable.getMin(),
+            'max'  : variable.getMax(),
+            'is_constant': variable.isConstant()
+        }
     return attributes
 
-def variable_collection_to_dataframe(var_collection:Union[list, ROOT.RooArgSet, ROOT.RooArgSet]):
+def variable_collection_to_dataframe(var_collection:Union[list, ROOT.RooArgSet, ROOT.RooArgSet],
+                                     asym_error:bool=False):
     import pandas as pd
     data = []
     for variable in var_collection:
-        attributes = get_variable_attributes(variable)
+        attributes = get_variable_attributes(variable, asym_error=asym_error)
         data.append(attributes)
     df = pd.DataFrame(data)
     return df
@@ -204,4 +216,19 @@ def get_correlation_matrix(roofit_result, lib:str="pandas"):
         return correlation_hist
     else:
         raise ValueError(f"unsupported library: {lib} (available options: \"root\", \"pandas\")")
-    
\ No newline at end of file
+        
+def get_relevant_components(target, reference:Set[ROOT.RooArgSet]):
+    result = ROOT.RooArgSet()
+    if target in reference:
+        result.add(target)
+    components = None
+    if isinstance(target, ROOT.RooAbsPdf):
+        components = target.getComponents()
+        components.remove(target)
+    elif isinstance(target, ROOT.RooProduct):
+        components = target.components()
+    if (components is not None) and components.getSize():
+        for component in components:
+            relevant_components = get_relevant_components(component, reference)
+            result.add(relevant_components)
+    return result
\ No newline at end of file
